#region Using declarations
using NinjaTrader.NinjaScript.DrawingTools;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Windows.Media;
using System.Xml.Serialization;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.Devside
{
    public class IchimokuCloud : Indicator
    {
        private Series<double> _conversionLine;
        private Series<double> _baseLine;
        private List<double> _savedHighs = new List<double>();
        private List<double> _savedLows = new List<double>();
        private int _lastBandCross;
        private bool _lastAHigher;
        private bool _AHigher;
        private int _drawingCounter = 1;
        private SolidColorBrush _trendColor;
        private SolidColorBrush _lastTrendColor;

        protected override void OnStateChange()
        {
            if (State == State.SetDefaults)
            {
                Description = @"Enter the description for your new custom Indicator here.";
                Name = "IchimokuCloud";
                Calculate = Calculate.OnBarClose;
                IsOverlay = false;
                DisplayInDataBox = true;
                DrawOnPricePanel = false;
                DrawHorizontalGridLines = true;
                DrawVerticalGridLines = true;
                PaintPriceMarkers = true;
                ScaleJustification = NinjaTrader.Gui.Chart.ScaleJustification.Right;
                //Disable this property if your indicator requires custom values that cumulate with each new market data event. 
                //See Help Guide for additional information.
                IsSuspendedWhileInactive = true;
                PlotLaggingSpan = true;
				PlotKenkanSen = true;
				PlotKijunSen = true;
				UseCloudColors = false;
                ShortPeriod = 9;
                MediumPeriod = 26;
                LongPeriod = 52;
                LaggingSpanOffset = 26;
                PositiveCloudColor = Brushes.Green;
                NegativeCloudColor = Brushes.Red;
                CloudOpacity = 40;
                AddPlot(Brushes.Green, "SenkouSpanA");
                AddPlot(Brushes.Red, "SenkouSpanB");
				AddPlot(Brushes.Blue, "Kenkan Sen");
				AddPlot(Brushes.Orange, "Kijun Sen");
                AddPlot(Brushes.Aqua, "LaggingSpan");

                //AddPlot(Brushes.Orange, "TestPlot");
            }
            else if (State == State.Configure)
            {
                //_conversionLine = new Series<double>(this, MaximumBarsLookBack.TwoHundredFiftySix);
                //_baseLine = new Series<double>(this, MaximumBarsLookBack.TwoHundredFiftySix);
                _lastBandCross = LongPeriod;
            }
        }

        // FormatPriceMarker method of a custom indicator
        public override string FormatPriceMarker(double price)
        {
            // Formats price marker values to 6 decimal places
            return price.ToString("N6");
        }

        protected override void OnBarUpdate()
        {
            //Update Lists
            _savedHighs.Add(High[0]);
            if (_savedHighs.Count > LongPeriod) _savedHighs.RemoveAt(0);
            _savedLows.Add(Low[0]);
            if (_savedLows.Count > LongPeriod) _savedLows.RemoveAt(0);

            //calculation starts after Lists are full
            if (CurrentBar >= LongPeriod)
            {
                //calculate intermediary values
                double shortHigh = _savedHighs.GetRange(LongPeriod - ShortPeriod, ShortPeriod).Max();
                double mediumHigh = _savedHighs.GetRange(LongPeriod - MediumPeriod, MediumPeriod).Max();
                double longHigh = _savedHighs.Max();

                double shortLow = _savedHighs.GetRange(LongPeriod - ShortPeriod, ShortPeriod).Min();
                double mediumLow = _savedHighs.GetRange(LongPeriod - MediumPeriod, MediumPeriod).Min();
                double longLow = _savedHighs.Min();

                KenkanSen[0] = (shortHigh + shortLow) / 2;
                KijunSen[0] = (mediumHigh + mediumLow) / 2;

                //calulate line values
                SenkouSpanA[0] = (KenkanSen[0] + KijunSen[0]) / 2;
                SenkouSpanB[0] = (longHigh + longLow) / 2;
                if (PlotLaggingSpan && CurrentBar > LaggingSpanOffset) LaggingSpan[LaggingSpanOffset] = Close[0];

                //determine color
                #region manage drawings
                if (Math.Abs(SenkouSpanA[0] - SenkouSpanB[0]) < 0.000000001)
                {
                    _trendColor = _lastTrendColor;
                    _AHigher = _lastAHigher;
                }
                else if (SenkouSpanA[0] >= SenkouSpanB[0])
                {
                    _trendColor = PositiveCloudColor;
                    _AHigher = true;
                }
                else
                {
                    _trendColor = NegativeCloudColor;
                    _AHigher = false;
                }
				
				if(UseCloudColors){
                PlotBrushes[0][0] = _trendColor;
                PlotBrushes[1][0] = _trendColor;
				}
				
                if (CurrentBar == LongPeriod) _lastAHigher = _AHigher;
                if (_AHigher != _lastAHigher)
                {                    
                    _lastBandCross = CurrentBar - 1;
                    _drawingCounter++;
                }
                Draw.Region(this, "IchimokuCloud Region " + _drawingCounter, 0, CurrentBar - _lastBandCross, SenkouSpanA, SenkouSpanB, null, _trendColor, CloudOpacity);

                _lastTrendColor = _trendColor;
                _lastAHigher = _AHigher;
                #endregion
            }
        }

        #region Properties
        [NinjaScriptProperty]
        [Display(Name = "Plot Kenkan Sen", Order = 10, GroupName = "Plotting")]
        public bool PlotKenkanSen
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Plot Kijun Sen", Order = 20, GroupName = "Plotting")]
        public bool PlotKijunSen
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Plot Lagging Span", Order = 30, GroupName = "Plotting")]
        public bool PlotLaggingSpan
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Positive Cloud Color", Order = 36, GroupName = "Plotting")]
        public SolidColorBrush PositiveCloudColor
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Negative Cloud Color", Order = 38, GroupName = "Plotting")]
        public SolidColorBrush NegativeCloudColor
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Cloud Opacity", Order = 39, GroupName = "Plotting")]
        public int CloudOpacity
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Use Cloud Colors for Leading Spans", Order = 40, GroupName = "Plotting")]
        public bool UseCloudColors
        { get; set; }
		
        [NinjaScriptProperty]
        [Display(Name = "Kenkan Sen Periode", Order = 20, GroupName = "Values")]
        public int ShortPeriod
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Kijun Sen Periode", Order = 30, GroupName = "Values")]
        public int MediumPeriod
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Senkou Span B Periode", Order = 40, GroupName = "Values")]
        public int LongPeriod
        { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Lagging Span Offset", Order = 50, GroupName = "Values")]
        public int LaggingSpanOffset
        { get; set; }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> SenkouSpanA
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> SenkouSpanB
        {
            get { return Values[1]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> LaggingSpan
        {
            get { return Values[2]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> KenkanSen
        {
            get { return Values[3]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> KijunSen
        {
            get { return Values[4]; }
        }
        #endregion

    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Devside.IchimokuCloud[] cacheIchimokuCloud;
		public Devside.IchimokuCloud IchimokuCloud(bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			return IchimokuCloud(Input, plotKenkanSen, plotKijunSen, plotLaggingSpan, positiveCloudColor, negativeCloudColor, cloudOpacity, useCloudColors, shortPeriod, mediumPeriod, longPeriod, laggingSpanOffset);
		}

		public Devside.IchimokuCloud IchimokuCloud(ISeries<double> input, bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			if (cacheIchimokuCloud != null)
				for (int idx = 0; idx < cacheIchimokuCloud.Length; idx++)
					if (cacheIchimokuCloud[idx] != null && cacheIchimokuCloud[idx].PlotKenkanSen == plotKenkanSen && cacheIchimokuCloud[idx].PlotKijunSen == plotKijunSen && cacheIchimokuCloud[idx].PlotLaggingSpan == plotLaggingSpan && cacheIchimokuCloud[idx].PositiveCloudColor == positiveCloudColor && cacheIchimokuCloud[idx].NegativeCloudColor == negativeCloudColor && cacheIchimokuCloud[idx].CloudOpacity == cloudOpacity && cacheIchimokuCloud[idx].UseCloudColors == useCloudColors && cacheIchimokuCloud[idx].ShortPeriod == shortPeriod && cacheIchimokuCloud[idx].MediumPeriod == mediumPeriod && cacheIchimokuCloud[idx].LongPeriod == longPeriod && cacheIchimokuCloud[idx].LaggingSpanOffset == laggingSpanOffset && cacheIchimokuCloud[idx].EqualsInput(input))
						return cacheIchimokuCloud[idx];
			return CacheIndicator<Devside.IchimokuCloud>(new Devside.IchimokuCloud(){ PlotKenkanSen = plotKenkanSen, PlotKijunSen = plotKijunSen, PlotLaggingSpan = plotLaggingSpan, PositiveCloudColor = positiveCloudColor, NegativeCloudColor = negativeCloudColor, CloudOpacity = cloudOpacity, UseCloudColors = useCloudColors, ShortPeriod = shortPeriod, MediumPeriod = mediumPeriod, LongPeriod = longPeriod, LaggingSpanOffset = laggingSpanOffset }, input, ref cacheIchimokuCloud);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Devside.IchimokuCloud IchimokuCloud(bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			return indicator.IchimokuCloud(Input, plotKenkanSen, plotKijunSen, plotLaggingSpan, positiveCloudColor, negativeCloudColor, cloudOpacity, useCloudColors, shortPeriod, mediumPeriod, longPeriod, laggingSpanOffset);
		}

		public Indicators.Devside.IchimokuCloud IchimokuCloud(ISeries<double> input , bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			return indicator.IchimokuCloud(input, plotKenkanSen, plotKijunSen, plotLaggingSpan, positiveCloudColor, negativeCloudColor, cloudOpacity, useCloudColors, shortPeriod, mediumPeriod, longPeriod, laggingSpanOffset);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Devside.IchimokuCloud IchimokuCloud(bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			return indicator.IchimokuCloud(Input, plotKenkanSen, plotKijunSen, plotLaggingSpan, positiveCloudColor, negativeCloudColor, cloudOpacity, useCloudColors, shortPeriod, mediumPeriod, longPeriod, laggingSpanOffset);
		}

		public Indicators.Devside.IchimokuCloud IchimokuCloud(ISeries<double> input , bool plotKenkanSen, bool plotKijunSen, bool plotLaggingSpan, SolidColorBrush positiveCloudColor, SolidColorBrush negativeCloudColor, int cloudOpacity, bool useCloudColors, int shortPeriod, int mediumPeriod, int longPeriod, int laggingSpanOffset)
		{
			return indicator.IchimokuCloud(input, plotKenkanSen, plotKijunSen, plotLaggingSpan, positiveCloudColor, negativeCloudColor, cloudOpacity, useCloudColors, shortPeriod, mediumPeriod, longPeriod, laggingSpanOffset);
		}
	}
}

#endregion
